@@ -1,5 +1,9 @@
 import os
 import sys
+import warnings
+
+warnings.filterwarnings("ignore", category=DeprecationWarning)
+warnings.filterwarnings("ignore", category=UserWarning)
 
 
 def main():
@@ -28,7 +32,7 @@ def main():
 
     import pear_ebi.tree_emb_parser
     from pear_ebi.calculate_distances import hashrf
-    from pear_ebi.embeddings import PCA_e, tSNE_e
+    from pear_ebi.embeddings import PCoA_e, tSNE_e
     from pear_ebi.interactive_mode import interactive
     from pear_ebi.tree_set import set_collection, tree_set
 
@@ -89,7 +93,7 @@ def main():
                 print("[orange1]\n- Leaving PEAR -")
                 exit()
             try:
-                exec(interactive.interact(control))
+                exec(interactive.interact(control), locals(), globals())
             except KeyboardInterrupt:
                 print("[red] Interrupted")
                 pass
@@ -233,6 +237,17 @@ def main():
             # with the value given in terminal
             # ─── Compute Distances ────────────────────────────────────────
             method = config["method"] if args.method is None else args.method
+            if method not in (
+                "hashrf_RF",
+                "hashrf_wRF",
+                "smart_RF",
+                "tqdist_quartet",
+                "tqdist_triplet",
+                None,
+            ):
+                sys.exit(
+                    "Invalid method - choose among: hashrf_RF, hashrf_wRF, smart_RF, tqdist_quartet, tqdist_triplet"
+                )
             if method is not None:
                 SET.calculate_distances(method)
 
@@ -246,7 +261,7 @@ def main():
                 method_embedding = (
                     config["embedding"]["method"]
                     if config["embedding"]["method"] is not None
-                    else "pca"
+                    else "pcoa"
                 )
                 dimensions = (
                     config["embedding"]["dimensions"]
@@ -280,9 +295,9 @@ def main():
             #    report = True
             report = False
 
-            if args.pca is not None:
-                method_embedding = "pca"
-                dimensions = args.pca
+            if args.pcoa is not None:
+                method_embedding = "pcoa"
+                dimensions = args.pcoa
             elif config["embedding"] is None and args.tsne is None:
                 method_embedding = None
                 dimensions = None
@@ -290,7 +305,7 @@ def main():
             if args.tsne is not None:
                 method_embedding = "tsne"
                 dimensions = args.tsne
-            elif config["embedding"] is None and args.pca is None:
+            elif config["embedding"] is None and args.pcoa is None:
                 method_embedding = None
                 dimensions = None
 
@@ -360,7 +375,7 @@ def main():
                     name_plot = config["plot"]["name"]
                     plot_meta = (
                         config["plot"]["plot_meta"]
-                        if config["plot"]["meta"] is not None
+                        if config["plot"]["plot_meta"] is not None
                         else "SET-ID"
                     )
                     plot_set = config["plot"]["plot_set"]
@@ -374,25 +389,26 @@ def main():
                         if config["plot"]["same_scale"] is not None
                         else False
                     )
-
                     show = (
                         config["plot"]["show"]
                         if config["plot"]["show"] is not None
                         else False
                     )
+                    z_axis = config["plot"]["z_axis"]
 
                     if dimensions > 2:
-                        name_plot = (
-                            name_plot + "3D"
+                        name_plot3d = (
+                            name_plot3d + "3D"
                             if name_plot is not None
                             else f"{method_embedding.upper()}_3D"
                         )
                         fig = SET.plot_3D(
                             method_embedding,
-                            name_plot=name_plot,
+                            name_plot=name_plot3d,
                             plot_meta=plot_meta,
                             plot_set=plot_set,
                             select=select,
+                            z_axis=z_axis,
                             same_scale=same_scale,
                             save=True,
                         )
@@ -400,14 +416,14 @@ def main():
                         if show:
                             fig.show()
 
-                    name_plot = (
+                    name_plot2d = (
                         name_plot + "2D"
                         if name_plot is not None
                         else f"{method_embedding.upper()}_2D"
                     )
                     fig = SET.plot_2D(
                         method_embedding,
-                        name_plot=name_plot,
+                        name_plot=name_plot2d,
                         plot_meta=plot_meta,
                         plot_set=plot_set,
                         select=select,
@@ -419,22 +435,32 @@ def main():
                         fig.show()
 
                 else:
+                    if "name_plot" not in globals() and "name_plot" not in locals():
+                        name_plot = None
+
                     if dimensions > 2:
-                        name_plot = f"{method_embedding.upper()}_3D"
+                        name_plot3d = (
+                            name_plot + "3D"
+                            if name_plot is not None
+                            else f"{method_embedding.upper()}_3D"
+                        )
                         fig = SET.plot_3D(
                             method_embedding,
-                            name_plot=name_plot,
+                            name_plot=name_plot3d,
                             save=True,
                         )
 
                         if args.plot:
                             fig.show()
 
-                        fig.show()
-                    name_plot = f"{method_embedding.upper()}_2D"
+                    name_plot2d = (
+                        name_plot + "2D"
+                        if name_plot is not None
+                        else f"{method_embedding.upper()}_2D"
+                    )
                     fig = SET.plot_2D(
                         method_embedding,
-                        name_plot=name_plot,
+                        name_plot=name_plot2d,
                         save=True,
                     )
 
@@ -449,6 +475,7 @@ def main():
 
     except KeyboardInterrupt:
         print("[orange1]\n- Leaving PEAR -")
+        return
 
     print("[orange1]\n- Leaving PEAR -")
 
