@@ -9,7 +9,7 @@ from .emb_quality import DRM, pear_correlation
 
 # ──────────────────────────────────────────────────────────────────────────────
 # ─── ISOMAP N COMPONENTS ──────────────────────────────────────────────────────
-def isomap(distance_matrix, n_components, metadata=None, quality=False, report=False):
+def isomap(distance_matrix, n_components, metadata=None, quality=False, report=False, output="./ISOMAP_Embedding.csv"):
     """embed distance_matrix in n_components with Isomap
 
     Args:
@@ -22,7 +22,7 @@ def isomap(distance_matrix, n_components, metadata=None, quality=False, report=F
     """
     embedding = Isomap(n_components=n_components)
     components = embedding.fit_transform(distance_matrix)
-    pd.DataFrame(components).to_csv("./ISOMAP_Embedding.csv", header=False, index=False)
+    pd.DataFrame(components).to_csv(output, header=False, index=False)
 
     if report:
         Xr = None
