@@ -37,7 +37,6 @@ import warnings
 import numpy as np
 import pandas as pd
 from rich import print
-
 # ? rich is a very nice library that allows to
 # ? easily format the output of console
 # ? https://github.com/Textualize/rich
@@ -59,7 +58,7 @@ from scipy.sparse import SparseEfficiencyWarning
 # importing other modules
 # try:
 from .calculate_distances import hashrf, maple_RF, tqdist
-from .embeddings import Isomap_e, LLE_e, PCA_e, tSNE_e
+from .embeddings import Isomap_e, LLE_e, PCoA_e, tSNE_e
 from .embeddings.graph import graph
 from .interactive_mode import interactive
 from .subsample import subsample
@@ -67,7 +66,6 @@ from .subsample import subsample
 # except:
 #    sys.exit("Error")
 
-
 warnings.filterwarnings("ignore", category=DeprecationWarning)
 warnings.filterwarnings("ignore", category=UserWarning)
 warnings.filterwarnings("ignore", category=SparseEfficiencyWarning)
@@ -95,9 +93,9 @@ class tree_set:
         self.output_file = output_file
         self.distance_matrix = distance_matrix
         self.metadata = metadata
-        self.embedding_pca2D = None
+        self.embedding_pcoa2D = None
         self.embedding_tsne2D = None
-        self.embedding_pca3D = None
+        self.embedding_pcoa3D = None
         self.embedding_tsne3D = None
 
         if self.output_file == None:
@@ -183,7 +181,7 @@ class tree_set:
         print(f"[bold blue]{method} | Done!")
 
     # ─── EMBED ─────────────────────────────────────────────────────────────────
-    def embed(self, method, dimensions, quality=False, report=False):
+    def embed(self, method, dimensions, quality=False, report=False, output=None):
         """Compute embedding with n-dimensions and method of choice
 
         Args:
@@ -192,7 +190,7 @@ class tree_set:
             quality (bool, optional): returns quality report and self.emb_quality. Defaults to False.
         """
         methods = {
-            "pca": PCA_e.pca,
+            "pcoa": PCoA_e.pcoa,
             "tsne": tSNE_e.tsne,
             "isomap": Isomap_e.isomap,
             "lle": LLE_e.lle,
@@ -202,20 +200,24 @@ class tree_set:
         if type(self.distance_matrix) == type(None):
             self.calculate_distances("hashrf_RF")
 
-        dim = dimensions if dimensions > 2 else 3
+        if dimensions < 2:
+            sys.exit("Dimensions of embedding must be greater or equal to 2")
 
+        if output == None:
+            output = f"./{os.path.splitext(os.path.basename(self.file))[0]}_{str.upper(method)}_embedding.csv"
         with self.console.status("[bold green]Embedding distances...") as status:
             embedding = methods[method](
                 self.distance_matrix,
-                dim,
+                dimensions,
                 self.metadata,
                 quality=quality if not report else True,
                 report=report,
+                output=output,
             )
         print(f"[bold blue]{method} | Done!")
 
         if quality:
-            if method == "pca":
+            if method == "pcoa":
                 embedding, var, corr, self.emb_quality = embedding
                 print(
                     f"With {dimensions} components/dimensions, the explained variance is {var:.2f},\n with an estimated correlation {corr[0, 1]:.2f} with the {self.n_trees}-dimensional coordinates"
@@ -226,10 +228,16 @@ class tree_set:
                     f"With {dimensions} components/dimensions, the estimated correlation with the {self.n_trees}-dimensional coordinates is {corr[0, 1]:.2f}"
                 )
 
-        if method == "pca":
-            self.embedding_pca = embedding
-            self.embedding_pca3D = embedding[:, :4]
-            self.embedding_pca2D = embedding[:, :3]
+        if method == "pcoa":
+            self.embedding_pcoa = embedding
+            if dimensions == 2:
+                self.embedding_pcoa2D = embedding
+            if dimensions == 3:
+                self.embedding_pcoa2D = embedding[:, :3]
+                self.embedding_pcoa3D = embedding
+            if dimensions > 3:
+                self.embedding_pcoa3D = embedding[:, :4]
+                self.embedding_pcoa2D = embedding[:, :3]
 
         elif method == "tsne":
             if dimensions > 3:
@@ -237,8 +245,14 @@ class tree_set:
                     "t-SNE with more than 3 dimensions can be considerably slow"
                 )
             self.embedding_tsne = embedding
-            self.embedding_tsne3D = embedding[:, :4]
-            self.embedding_tsne2D = embedding[:, :3]
+            if dimensions == 2:
+                self.embedding_tsne2D = embedding
+            if dimensions == 3:
+                self.embedding_tsne2D = embedding[:, :3]
+                self.embedding_tsne3D = embedding
+            if dimensions > 3:
+                self.embedding_tsne3D = embedding[:, :4]
+                self.embedding_tsne2D = embedding[:, :3]
 
         elif method == "isomap":
             if dimensions > 3:
@@ -246,15 +260,27 @@ class tree_set:
                     "Isomap with more than 3 dimensions can be considerably slow"
                 )
             self.embedding_isomap = embedding
-            self.embedding_isomap3D = embedding[:, :4]
-            self.embedding_isomap2D = embedding[:, :3]
+            if dimensions == 2:
+                self.embedding_isomap2D = embedding
+            if dimensions == 3:
+                self.embedding_isomap2D = embedding[:, :3]
+                self.embedding_isomap3D = embedding
+            if dimensions > 3:
+                self.embedding_isomap3D = embedding[:, :4]
+                self.embedding_isomap2D = embedding[:, :3]
 
         elif method == "lle":
             if dimensions > 3:
                 warnings.warn("LLE with more than 3 dimensions can be considerably slow")
             self.embedding_lle = embedding
-            self.embedding_lle3D = embedding[:, :4]
-            self.embedding_lle2D = embedding[:, :3]
+            if dimensions == 2:
+                self.embedding_lle2D = embedding
+            if dimensions == 3:
+                self.embedding_lle2D = embedding[:, :3]
+                self.embedding_lle3D = embedding
+            if dimensions > 3:
+                self.embedding_lle3D = embedding[:, :4]
+                self.embedding_lle2D = embedding[:, :3]
 
     # ─── PLOT EMBEDDING ─────────────────────────────────────────────────────────
 
@@ -282,7 +308,7 @@ class tree_set:
             same_scale (bool, optional): use same color_scale for all traces when scale is continuous. Defaults to False.
 
         Raises:
-            ValueError: method can only be either pca or tsne for now
+            ValueError: method can only be either pcoa or tsne for now
 
         Returns:
             plot: either interactive or not
@@ -291,13 +317,13 @@ class tree_set:
         # you can surely write something better here @andrear
         if type(plot_set) == type(None):
             plot_set = self.sets
-        if method == "pca":
+        if method == "pcoa":
             if name_plot == None:
-                name_plot = "PCA_2D"
-            if type(self.embedding_pca2D) == type(None):
-                self.embed("pca", 2)
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_PCOA_2D"
+            if type(self.embedding_pcoa2D) == type(None):
+                self.embed("pcoa", 2)
             fig = graph.plot_embedding(
-                self.embedding_pca2D,
+                self.embedding_pcoa2D,
                 self.metadata,
                 2,
                 save,
@@ -311,7 +337,7 @@ class tree_set:
 
         elif method == "tsne":
             if name_plot == None:
-                name_plot = "TSNE_2D"
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_TSNE_2D"
             if type(self.embedding_tsne2D) == type(None):
                 self.embed("tsne", 2)
             fig = graph.plot_embedding(
@@ -329,7 +355,7 @@ class tree_set:
 
         elif method == "isomap":
             if name_plot == None:
-                name_plot = "ISOMAP_2D"
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_ISOMAP_2D"
             if type(self.embedding_isomap2D) == type(None):
                 self.embed("isomap", 2)
             fig = graph.plot_embedding(
@@ -347,7 +373,7 @@ class tree_set:
 
         elif method == "lle":
             if name_plot == None:
-                name_plot = "LLE_2D"
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_LLE_2D"
             if type(self.embedding_lle2D) == type(None):
                 self.embed("lle", 2)
             fig = graph.plot_embedding(
@@ -364,7 +390,7 @@ class tree_set:
             )
 
         else:
-            raise ValueError("'method' can only be either 'pca' or 'tsne' ")
+            raise ValueError("'method' can only be either 'pcoa' or 'tsne' ")
 
         return fig
 
@@ -393,20 +419,20 @@ class tree_set:
             same_scale (bool, optional): use same color_scale for all traces when scale is continuous. Defaults to False.
 
         Raises:
-            ValueError: method can only be either pca or tsne for now
+            ValueError: method can only be either pcoa or tsne for now
 
         Returns:
             plot: either interactive or not
         """
         if type(plot_set) == type(None):
             plot_set = self.sets
-        if method == "pca":
+        if method == "pcoa":
             if name_plot == None:
-                name_plot = "PCA_3D"
-            if type(self.embedding_pca3D) == type(None):
-                self.embed("pca", 3)
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_PCOA_3D"
+            if type(self.embedding_pcoa3D) == type(None):
+                self.embed("pcoa", 3)
             fig = graph.plot_embedding(
-                self.embedding_pca3D,
+                self.embedding_pcoa3D,
                 self.metadata,
                 3,
                 save,
@@ -421,7 +447,7 @@ class tree_set:
 
         elif method == "tsne":
             if name_plot == None:
-                name_plot = "TSNE_3D"
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_TSNE_3D"
             if type(self.embedding_tsne3D) == type(None):
                 self.embed("tsne", 3)
             fig = graph.plot_embedding(
@@ -440,7 +466,7 @@ class tree_set:
 
         elif method == "isomap":
             if name_plot == None:
-                name_plot = "ISOMAP_3D"
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_ISOMAP_3D"
             if type(self.embedding_isomap3D) == type(None):
                 self.embed("isomap", 3)
             fig = graph.plot_embedding(
@@ -459,7 +485,7 @@ class tree_set:
 
         elif method == "lle":
             if name_plot == None:
-                name_plot = "LLE_3D"
+                name_plot = f"./{os.path.splitext(os.path.basename(self.file))[0]}_LLE_3D"
             if type(self.embedding_lle3D) == type(None):
                 self.embed("lle", 3)
             fig = graph.plot_embedding(
@@ -477,7 +503,7 @@ class tree_set:
             )
 
         else:
-            raise ValueError("'method' can only be either 'pca' or 'tsne' ")
+            raise ValueError("'method' can only be either 'pcoa' or 'tsne' ")
 
         return fig
 
@@ -493,6 +519,10 @@ class tree_set:
         Returns:
             subset plots: 2D and 3D embedding plots of subset
         """
+        if isinstance(self, set_collection):
+            files = [TS.file for TS in self.collection]
+        else:
+            files = [self.file]
         console = Console()
         with console.status("[bold blue]Extracting subsample...") as status:
             if method == "syst":
@@ -500,26 +530,31 @@ class tree_set:
                     command = [
                         "pypy3",
                         f"{current}/subsample/subsample.py",
-                        self.file,
+                        str(files),
                         str(self.n_trees),
                         str(n_required),
                     ]
-                    res = subprocess.check_output(command, universal_newlines=True).split(
-                        "\n"
-                    )
+                    res = subprocess.check_output(command, universal_newlines=True).split("\n")
                     subsample_trees, idxs = eval(res[3]), eval(res[4])
+                    
                 else:
                     console.log(
                         "[bold red]Could not find pypy3 on your sytem PATH - using python3..."
                     )
                     subsample_trees, idxs = subsample.subsample(
-                        self.file, self.n_trees, n_required, subp=False
+                        str(files), self.n_trees, n_required, subp=False
                     )
 
             else:
-                with open(self.file, "r") as f:
-                    trees = list(enumerate(f.readlines()))
-                    f.close()
+                trees = list()
+                last_max = 0
+                for file in files:
+                    with open(file, "r") as f:
+                        trees_file = list(f.readlines())
+                        n_t = len(trees_file)
+                        trees.extend(list(enumerate(trees_file, start=last_max)))
+                        last_max += n_t
+                        f.close()
 
                 if method == "random":
                     selection = random.sample(trees, n_required)
@@ -534,7 +569,7 @@ class tree_set:
                 else:
                     sys.exit(f"Method {method} not available for subsampling")
 
-            file_sub = f"SUBSAMPLE"
+            file_sub = f"{self.file}_SUBSAMPLE"
             with open(file_sub, "w") as f:
                 for i in subsample_trees:
                     f.write(i)
@@ -542,7 +577,7 @@ class tree_set:
             # print(len(subsample_trees), len(idxs))
             status.update("[bold green]Calculating distances...")
             dM = hashrf.hashrf(file_sub, n_required, file_sub + "_distances.csv")
-            components = PCA_e.pca(dM, 3)
+            components = PCoA_e.pcoa(dM, 3)
             status.update(f"[bold blue] Done!")
             time.sleep(0.2)
 
@@ -582,15 +617,15 @@ class set_collection(tree_set):
         output_file: facultative - specifies output_file of distance matrix"""
 
         self.id = uuid.uuid4()
-        self.file = file + str(self.id)
+        self.file = file + str(self.id) if file == "Set_collection_" else file
         self.distance_matrix = (
             pd.read_csv(distance_matrix, header=None, index_col=None).values  #
             if distance_matrix
             else distance_matrix
         )
-        self.embedding_pca2D = None
+        self.embedding_pcoa2D = None
         self.embedding_tsne2D = None
-        self.embedding_pca3D = None
+        self.embedding_pcoa3D = None
         self.embedding_tsne3D = None
 
         if self.file != "Set_collection_" + str(self.id) and output_file is None:
@@ -607,12 +642,13 @@ class set_collection(tree_set):
 
         if isinstance(collection, tree_set):
             self.collection = [collection]
-            with open(self.file, "w") as trees:
-                with open(collection.file, "r") as file:
-                    trees.write(file.read())
-                    file.close()
-                trees.close()
+            #with open(self.file, "w") as trees:
+            #    with open(collection.file, "r") as file:
+            #        trees.write(file.read())
+            #        file.close()
+            #    trees.close()
 
+        
         elif len(collection) > 0:
             remove = list()
             for i, element in enumerate(collection):
@@ -644,7 +680,17 @@ class set_collection(tree_set):
 
         self.data = dict()
 
+        metadata_input = None
+        if type(metadata) != type(None):
+            try:
+                metadata_input = pd.read_csv(metadata, index_col=0)
+            except:
+                sys.exit(
+                    "There's an error with the Metadata file - please check the correct location and name of the .csv file"
+                )
+
         self.metadata = pd.DataFrame()
+
         self.n_trees = 0
         for set in self.collection:
             key = os.path.splitext(os.path.basename(set.file))[0]
@@ -661,6 +707,14 @@ class set_collection(tree_set):
 
         self.metadata.reset_index(drop=True, inplace=True)
 
+        if type(metadata) != type(None):
+            try:
+                self.metadata = pd.concat([self.metadata, metadata_input], axis=1)
+            except:
+                sys.exit(
+                    "There's an error with the Metadata file - please check that the .csv file is compatible with the set_collection structure"
+                )
+
         self.sets = np.unique(self.metadata["SET-ID"])
 
     # ─── CALCULATE DISTANCES ───────────────────────────────────────────────────
@@ -679,7 +733,7 @@ class set_collection(tree_set):
             "None": None,
         }
 
-        if method in ("hashrf_RF", "hashrf_wRF", "tqdist_quartet", "tqdist_triplet"):
+        if method in ("hashrf_RF", "hashrf_wRF", "smart_RF", "tqdist_quartet", "tqdist_triplet"):
             with open(self.file, "w") as trees:
                 for set in self.collection:
                     with open(set.file, "r") as file:
